@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Config;
 
-our $VERSION = '2.000017';
+our $VERSION = '2.000018';
 $VERSION = eval $VERSION;
 
 BEGIN {
@@ -554,6 +554,10 @@ sub build_fish_env_declaration {
     return qq{set -e $name;\n};
   }
 
+  # fish has special handling for PATH, CDPATH, and MANPATH.  They are always
+  # treated as arrays, and joined with ; when storing the environment.  Other
+  # env vars can be arrays, but will be joined without a separator.  We only
+  # really care about PATH, but might as well make this routine more general.
   if ($name =~ /^(?:CD|MAN)?PATH$/) {
     $value =~ s/$_path_sep/ /g;
     my $silent = $name =~ /^(?:CD)?PATH$/ ? " ^"._devnull : '';
@@ -658,24 +662,31 @@ sub resolve_relative_path {
 sub ensure_dir_structure_for {
   my ($class, $path, $opts) = @_;
   $opts ||= {};
-  my %dirs;
-  my @dirs = grep ! -d, (
+  my @dirs;
+  foreach my $dir (
     $class->lib_paths_for($path),
     $class->install_base_bin_path($path),
-  );
-  @dirs{@dirs} = ();
-  for my $dir (@dirs) {
-    require File::Basename;
-    my $parent = File::Basename::dirname($dir);
-    unless (exists $dirs{$parent} || -d $parent) {
-      $dirs{$parent} = undef;
-      push @dirs, $parent
+  ) {
+    my $d = $dir;
+    while (!-d $d) {
+      push @dirs, $d;
+      require File::Basename;
+      $d = File::Basename::dirname($d);
     }
   }
+
   warn "Attempting to create directory ${path}\n"
     if !$opts->{quiet} && @dirs;
-  mkdir $_ or die "Unable to create $_: $!"
-    for reverse @dirs;
+
+  my %seen;
+  foreach my $dir (reverse @dirs) {
+    next
+      if $seen{$dir}++;
+
+    mkdir $dir
+      or -d $dir
+      or die "Unable to create $dir: $!"
+  }
   return;
 }
 
